# -*- coding: utf-8 -*-

import boto3
from botocore.exceptions import ClientError
import click
from dotenv import load_dotenv
import json
import logging
import os
import sys

from epiclient import Epiclient
from epipearl.epipearl import Epipearl


class AwsS3(object):

    def __init__(self, bucket_name):
        self.s3 = boto3.resource('s3')

        try:
            self.s3.meta.client.head_bucket(Bucket=bucket_name)
            self.bucket = self.s3.Bucket(bucket_name)
        except ClientError as e:
            click.echo('bucket({}) not found or not accessible'.format(
                bucket_name))
            raise e

    @property
    def bucket_name(self):
        return self.bucket.name

    def get_device_settings(self, device_name):
        try:
            key = '{}.json'.format(device_name)
            obj = self.bucket.Object(key).get()
            return obj['Body'].read()
        except ClientError:
            click.echo('object({}) not found in bucket({})'.format(
                key, self.bucket.name))
            return None

    def set_device_settings(self, device_name, settings):
        self.bucket.put_object(Key='{}.json'.format(device_name), Body=settings)


@click.command()
@click.argument('device_name', type=click.STRING)
@click.option(
    '--step', type=click.Choice([
        'validate', 'clear_channels', 'config_deinterlacing', 'create_channels',
        'config_layout', 'config_encodings', 'config_recorder_channels',
        'config_recorder_settings', 'config_live', 'config_general',
        'config_mhpearl']),
    help='when absent execute all steps')
@click.option('--verbose/--quiet', 'verbose', default=False)
def main(
        device_name,
        step, verbose):
    """Console script for epiclient"""

    # set environment for main()
    dotenv_filepath = os.path.expanduser('~/.epiclient.env')
    resp = load_dotenv(dotenv_filepath)
    if resp is None:
        click.echo('missing `~/.epiclient.env` file!`')
        exit(1)


    EPI_USR = os.getenv('EPIPEARL_USER')
    EPI_PWD = os.getenv('EPIPEARL_PASSWD')
    MH_USR = os.getenv('MH_DIGEST_USER')
    MH_PWD = os.getenv('MH_DIGEST_PASSWD')
    RTMP_USR = os.getenv('RTMP_USER')
    RTMP_PWD = os.getenv('RTMP_PASSWD')
    CA_BUCKET = os.getenv('CA_SETTINGS_BUCKET')

    # log goes to stdout
    if verbose:
        logging.basicConfig(stream=sys.stdout, level=logging.DEBUG)
    else:
        logging.basicConfig(stream=sys.stdout, level=logging.WARN)


    # connect with s3 to get json configs
    try:
        s3 = AwsS3(CA_BUCKET)
    except ClientError as e:
        click.echo('error connecting to s3: {}'.format(e.message))
        exit(1)
    datastring = s3.get_device_settings(device_name)
    if datastring is None:
        click.echo('check settings for device({})'.format(device_name))
        exit(2)

    if Epiclient.validate_json_input(datastring):
        click.echo('input({}) is valid'.format(device_name))

    configs = json.loads(datastring)
    epiphan_url = configs['ca_url']

    if EPI_USR is None or EPI_PWD is None:
        click.echo('missing epiphan user/password')
        exit(1)

    click.echo('init epipearl({}, {}, ***)'.format(epiphan_url, EPI_USR))
    epicli = Epipearl(epiphan_url, EPI_USR, EPI_PWD)

    if step == 'validate':
        return 0

    if step is None or step == 'clear_channels':
        click.echo('----- clearing channels...')
        Epiclient.clear_channels(epicli, configs)
        click.echo('----- channels cleared')

    if step is None or step == 'config_deinterlacing':
        click.echo('----- configuring deinterlace...')
        Epiclient.configure_deinterlacing_sources(epicli, configs)
        click.echo('----- deinterlace configure')

    if step is None or step == 'create_channels':
        click.echo('----- creating channels...')
        updated_settings = Epiclient.create_channels(epicli, configs)
        click.echo('----- channels created')

        s3.set_device_settings(
                device_name,
                json.dumps(updated_settings, sort_keys=True, indent=4))
        click.echo('----- updated settings into s3')

    if step is None or step == 'config_encodings':
        click.echo('----- configuring channels encodings')
        Epiclient.configure_channel_encodings(epicli, configs)
        click.echo('----- encodings configured')

    if step is None or step == 'config_layout':
        click.echo('----- configuring channels layout...')
        Epiclient.configure_channel_layouts(epicli, configs)
        click.echo('----- layout configured')

    if step is None or step == 'config_live':
        if RTMP_USR is None or RTMP_PWD is None:
            click.echo('missing rtmp user/password')
            exit(2)
        click.echo('----- configuring channels live...')
        Epiclient.configure_live_streaming(epicli, configs, RTMP_USR, RTMP_PWD)
        click.echo('----- live configured')

    if step is None or step == 'config_recorder_channels':
        click.echo('----- configuring recorder channels')
        Epiclient.configure_recorder_channels(epicli, configs)
        click.echo('----- recorder channels configured')

    if step is None or step == 'config_recorder_settings':
        click.echo('----- configuring recorder settings')
        Epiclient.configure_recorder_settings(epicli, configs)
        click.echo('----- recorder settings configured')

    if step is None or step == 'config_mhpearl':
        if MH_USR is None or MH_PWD is None:
            click.echo('missing mh digest user/password')
            exit(3)
        click.echo('----- configuring mhpearl...')
        Epiclient.configure_mhpearl(epicli, configs, MH_USR, MH_PWD)
        click.echo('----- mhpearl configured')

    if step is None or step == 'config_general':
        click.echo('----- configuring general settings...')
        Epiclient.configure_general_settings(epicli, configs)
        click.echo('----- general settings configured')


#
# main
#
if __name__ == "__main__":

    main()



# -*- coding: utf-8 -*-

import os

cwd = os.getcwd()


def pytest_addoption(parser):
    parser.addoption(
            '--runlive', action='store_true',
            help='test with live capture agents')


def read_datafile(filename):
    filename = os.path.join(cwd, 'tests/json/%s.json' % filename)
    data = ''
    with open(filename, 'r') as myfile:
        data = myfile.read()

    return data.rstrip()  # read() adds a newline at the end of the file??



# -*- coding: utf-8 -*-

"""
epiclient
---------------
epipearl client to configure device as dce capture-agent for opencast
"""

import copy
from jinja2 import Template
import json
import jsonspec.validators
import logging
import os
import pkg_resources
import requests

from epipearl.errors import EpipearlError


logger = logging.getLogger(__name__)


class EpiclientError(Exception):
    """base class for epiclient module errors."""


class Epiclient(object):
    _role_settings_schema_filename = 'role_settings_schema.json'

    _layout_single_channel_template = Template('''{
    "audio": [
        {
            "settings": {
                "source": "{{source_id}}.{{aconnector}}-{{ainput}}-audio"
            },
            "type": "source"
        }
    ],
    "background": "#000000",
    "nosignal": {
        "id": "default"
    },
    "video": [
        {
            "position": {
                "height": "100%",
                "keep_aspect_ratio": true,
                "left": "0%",
                "top": "0%",
                "width": "100%"
            },
            "settings": {
                "source": "{{source_id}}.{{vconnector}}-{{vinput}}"
            },
            "type": "source"
        }
    ]
}
''')

    _layout_combined_channels_template = Template('''{
    "audio": [
        {
            "settings": {
                "source": "{{source_id}}.{{pr_aconnector}}-{{pr_ainput}}-audio"
            },
            "type": "source"
        }
    ],
    "nosignal": {
        "id": "default"
    },
    "background": "#000000",
    "video": [
        {
            "crop": {},
            "position": {
                "keep_aspect_ratio": true,
                "height": "100%",
                "width": "50%",
                "left": "0%",
                "top": "0%"
            },
            "settings": {
                "source": "{{source_id}}.{{pr_vconnector}}-{{pr_vinput}}"
            },
            "type": "source"
        },
        {
            "crop": {},
            "position": {
                "keep_aspect_ratio": true,
                "height": "100%",
                "width": "50%",
                "left": "50%",
                "top": "0%"
            },
            "settings": {
                "source": "{{source_id}}.{{pn_vconnector}}-{{pn_vinput}}"
            },
            "type": "source"
        }
    ]
}''')

    @classmethod
    def get_json_schema_fullpath(cls, json_schema_filename):
        """return the fullpath in the package, for the json schema."""
        resource_path = os.path.join('json_schema', json_schema_filename)
        return pkg_resources.resource_filename(__name__, resource_path)


    @classmethod
    def validate_json_input(cls, json_inputstring):
        """validates a json against role_settings schema."""

        filename = cls.get_json_schema_fullpath(
                cls._role_settings_schema_filename)
        with open(filename, 'r') as myfile:
            schema_string = myfile.read()

        json_data = json.loads(json_inputstring)
        json_schema = json.loads(schema_string)

        try:
            validator = jsonspec.validators.load(json_schema)
        except jsonspec.validators.CompilationError as e:
            msg = 'error loading schema: {}'.format(e.message)
            logger.error(msg)
            raise e

        try:
            resp = validator.validate(json_data)
        except jsonspec.validators.ValidationError as e:
            msg = 'error validating againts json_schema({}): {}'.format(
                    filename, e.message)
            logger.error(msg)
            raise e

        return True


    @classmethod
    def fetch_infocfg(cls, client):
        """get infocfg from capture agent."""
        try:
            r_infocfg = client.get_infocfg()
        except Exception as e:
            msg = 'failed to get_infocfg for device({}) - {}'.format(
                    client.url, e.message)
            logger.error(msg)
            raise e
        else:
            return r_infocfg


    @classmethod
    def clear_channels(cls, client, settings):
        """delete all dce channels and recorders for given epipearl client."""
        for rec_name in settings['recorders']:
            try:
                client.delete_recorder(
                        settings['recorders'][rec_name]['recorder_id'])
            except EpipearlError as e:
                msg = 'failed during clear recorder('
                msg += '{})({}) from epiphan({}) - {}'. format(
                        rec_name,
                        settings['recorders'][rec_name]['recorder_id'],
                        client.url, e.message)
                logger.error(msg)
                raise e

        for channel_name in settings['channels']:
            try:
                client.delete_channel(
                        settings['channels'][channel_name]['channel_id'])
            except EpipearlError as e:
                msg = 'failed during clear channel('
                msg += '{})({}) from epiphan({}) - {}'.format(
                        channel_name,
                        settings['channels'][channel_name]['channel_id'],
                        client.url, e.message)
                logger.error(msg)
                raise e

        return True


    @classmethod
    def configure_deinterlacing_sources(
            cls, client, settings, sources=None):
        """enabled/disable deinterlacing in video sources."""
        if sources is None:
            infocfg = cls.fetch_infocfg(client)
            if len(infocfg['sources']) > 0:
                sources = infocfg['sources']
            else:
                msg = 'failed to deinterlacing: no sources found'
                msg += ' in infocfg response for device({})'.format(
                        client.url)
                logger.error(msg)
                raise EpiclientError(msg)

        deinterlacing = settings['source_deinterlacing'] == 'on'
        for s in sources:
            if 'audio' not in s['name'].lower():
                try:
                    client.set_deinterlacing_source(
                            source_name=s['id'],
                            deinterlacing=deinterlacing)
                except EpipearlError as e:
                    msg = 'failed to deinterlacing source('
                    msg += '{}) in epiphan({}) - {}'.format(
                            s['id'], client.url, e.message)
                    logger.error(msg)
                    raise e

        return True


    @classmethod
    def create_channels(cls, client, settings):
        """create dce channels and recorders for given epipearl client."""
        for channel_name in settings['channels']:
            try:
                channel_id = client.create_channel(channel_name)
            except EpipearlError as e:
                msg = 'failed to create channel({}) - {}'.format(
                        channel_name, e.message)
                logger.error(msg)
                raise e
            else:
                settings['channels'][channel_name]['channel_id'] = channel_id

        for rec_name in settings['recorders']:
            try:
                rec_id = client.create_recorder(rec_name)
            except EpipearlError as e:
                msg = 'failed to create recorder({}) - {}'.format(
                        rec_name, e.message)
                logger.error(msg)
                raise e
            else:
                settings['recorders'][rec_name]['recorder_id'] = rec_id

        return settings


    @classmethod
    def _set_channel_layout(
            cls, client,
            channel_name, channel_id,
            layout):
        try:
            client.set_channel_layout(
                    channel_id=channel_id,
                    layout=layout)
        except EpipearlError as e:
            msg = 'failed to set layout for channel({})({}) - {}'.format(
                    channel_name, channel_id, e.message)
            logger.error(msg)
            raise e
        else:
            return True


    @classmethod
    def configure_channel_layouts(cls, client, settings):
        """set layouts for dce channels."""
        pr_ch = settings['channels']['dce_pr']
        pn_ch = settings['channels']['dce_pn']

        # presenter channel
        l = cls._layout_single_channel_template.render(
                source_id=settings['ca_capture_card_id'],
                vconnector=settings['pr_vconnector'],
                vinput=settings['pr_vinput'],
                aconnector=settings['pr_vconnector'],
                ainput=settings['pr_vinput'])
        layout = l.replace(' ', '').replace('\n', '')
        cls._set_channel_layout(
                client=client,
                channel_name='dce_pr', channel_id=pr_ch['channel_id'],
                layout=layout)

        # presentation channel
        l = cls._layout_single_channel_template.render(
                source_id=settings['ca_capture_card_id'],
                vconnector=settings['pn_vconnector'],
                vinput=settings['pn_vinput'],
                aconnector=settings['pr_vconnector'],  # audio from pr
                ainput=settings['pr_vinput'])
        layout = l.replace(' ', '').replace('\n', '')
        cls._set_channel_layout(
                client=client,
                channel_name='dce_pn', channel_id=pn_ch['channel_id'],
                layout=layout)

        # combined source channels
        l = cls._layout_combined_channels_template.render(
                source_id=settings['ca_capture_card_id'],
                pr_vconnector=settings['pr_vconnector'],
                pr_vinput=settings['pr_vinput'],
                pn_vconnector=settings['pn_vconnector'],
                pn_vinput=settings['pn_vinput'],
                pr_aconnector=settings['pr_vconnector'],
                pr_ainput=settings['pr_vinput'])
        layout = l.replace(' ', '').replace('\n', '')
        for channel_name in ['dce_live', 'dce_live_lowbr']:
            channel_id = settings['channels'][channel_name]['channel_id']
            cls._set_channel_layout(
                    client=client,
                    channel_name=channel_name,
                    channel_id=channel_id,
                    layout=layout)
        return True


    @classmethod
    def _set_encodings(
            cls, client, channel_id, channel_name,
            encoding_settings):
        try:
            resp = client.set_params(
                    channel=channel_id, params=encoding_settings)
        except (requests.HTTPError,
                requests.RequestException,
                requests.ConnectionError,
                requests.Timeout) as e:
            msg = 'failed to configure encodings for channel({})({})'.format(
                    channel_name, channel_id)
            msg += '- failed to call {}/set_params - {}'.format(
                    client.url,  e.message)
            logger.error(msg)
            raise e
        else:
            return True


    @classmethod
    def configure_channel_encodings(cls, client, settings):
        """set channel encodings for dce channels."""
        for (channel_name, ch) in settings['channels'].items():
            # merge common and specific encodings for this channel
            encodings = copy.copy(settings['channel_encodings'])
            encodings.update(ch['encodings'])

            cls._set_encodings(
                    client=client, channel_id=ch['channel_id'],
                    channel_name=channel_name,
                    encoding_settings=encodings)

    @classmethod
    def _set_live(
            cls, client, channel_id, channel_name,
            rtmp_url, rtmp_stream, rtmp_usr, rtmp_pwd):
        try:
            client.set_channel_rtmp(
                    channel_id=channel_id,
                    rtmp_url=rtmp_url,
                    rtmp_stream=rtmp_stream,
                    rtmp_usr=rtmp_usr,
                    rtmp_pwd=rtmp_pwd)
        except EpipearlError as e:
            msg = 'failed to configure rtmp for channel({})({}) - {}'.format(
                    channel_id, channel_name, e.message)
            logger.error(msg)
            raise e
        else:
            return True


    @classmethod
    def start_live(cls, client, channel_id, startlive=True):
        params = {'publish_type': '6'}
        if not startlive:
            params['publish_type'] = 0

        try:
            client.set_params(channel_id, params)
        except EpipearlError as e:
            msg = 'failed to {} live channel({}) in device({}) - {}'.format(
                    'start' if startlive else 'stop',
                    channel_id, client.url, e.message)
            logger.error(msg)
            raise e
        else:
            return True


    @classmethod
    def configure_live_streaming(
            cls, client, settings,
            rtmp_usr=None, rtmp_pwd=None):
        """set rtmp configs for dce live channels."""
        for (channel_name, ch) in settings['channels'].items():
            if 'live' in channel_name:
                try:
                    cls._set_live(
                        client,
                        channel_id=ch['channel_id'],
                        channel_name=channel_name,
                        rtmp_url=ch['rtmp_url'],
                        rtmp_stream=ch['stream_name'],
                        rtmp_usr=rtmp_usr,
                        rtmp_pwd=rtmp_pwd)
                except EpipearlError as e:
                    msg = 'failed to configure live in channel({}:{})'.format(
                            channel_name, ch['channel_id'])
                    logger.error(msg)
                    raise e

                # set streaming non-stop
                cls.start_live(client, ch['channel_id'])
        return True


    @classmethod
    def configure_recorder_channels(cls, client, settings):
        """set dce_pr and dce_pn as recorder channels."""
        channel_list = [
                settings['channels']['dce_pr']['channel_id'],
                settings['channels']['dce_pn']['channel_id']
                ]

        try:
            client.set_recorder_channels(
                    settings['recorders']['dce_prpn']['recorder_id'],
                    channel_list)
        except EpipearlError as e:
            msg = 'failed to set channels({}) in recorder({}) - {}'.format(
                    channel_list,
                    settings['recorders']['dce_prpn']['recorder_id'],
                    e.message)
            logger.error(msg)
            raise e
        return True


    @classmethod
    def configure_recorder_settings(cls, client, settings):
        prpn = settings['recorders']['dce_prpn']
        try:
            client.set_recorder_settings(
                    prpn['recorder_id'],
                    recording_timelimit_in_minutes=prpn['timelimit'],
                    recording_sizelimit_in_kbytes=prpn['sizelimit'],
                    output_format=prpn['output_format'])
        except EpipearlError as e:
            msg = 'failed to set recorder(dce_prpn:{}} settings - {}'.format(
                    prpn['recorder_id'], e.message)
            logger.error(msg)
            raise e
        return True


    @classmethod
    def configure_general_settings(cls, client, settings):
        """ntp, touchscreen, permanent_logs."""
        try:
            client.set_ntp(
                server=settings['date_and_time']['ntp_server'],
                timezone=settings['date_and_time']['timezone'])
        except EpipearlError as e:
            msg = 'failed to configure date_and_time for epiphan({}) - {}'.\
                    format(client.url, e.message)
            logger.error(msg)
            raise e

        try:
            client.set_permanent_logs(
                    log_enabled=settings[
                        'maintenance']['permanent_logs'] == 'on')
        except EpipearlError as e:
            msg = 'failed to configure permanent logs for epiphan({}) - {}'.\
                    format(client.url, e.message)
            logger.error(msg)
            raise e

        try:
            client.set_touchscreen(
                    screen_timeout=settings[
                        'touchscreen']['episcreen_timeout'])
        except EpipearlError as e:
            msg = 'failed to configure touchscreen for epiphan({}) - {}'.\
                    format(client.url, e.message)
            logger.error(msg)
            raise e

        return True


    @classmethod
    def configure_mhpearl(
            cls, client, settings,
            mh_digest_usr=None, mh_digest_pwd=None):
        try:
            client.set_mhpearl_settings(
                    device_name=settings['mh_ca_name'],
                    device_channel=settings[
                        'recorders']['dce_prpn']['recorder_id'],
                    file_search_range_in_seconds=settings[
                        'mhpearl_file_search_range'],
                    admin_server_url=settings['mh_admin_url'],
                    admin_server_usr=mh_digest_usr,
                    admin_server_pwd=mh_digest_pwd,
                    update_frequency_in_seconds=settings[
                        'mhpearl_update_frequency'],
                    backup_agent=(settings['role'] == 'secondary'))
        except EpipearlError as e:
            msg = 'failed to configure mhpear for epiphan({}) - {}'.\
                    format(client.url, e.message)
            logger.error(msg)
            raise e
        else:
            return True



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup
from setuptools import find_packages

import os
import re
import codecs


def read_version():
    version_file = codecs.open(
            os.path.join(
                os.path.abspath(os.path.dirname(__file__)),
                'epiclient/__init__.py'),
            'r', 'utf-8').read()
    return re.search(
            r"^__version__ = ['\"]([^'\"]*)['\"]",
            version_file, re.M).group(1)

version = read_version()


with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

requirements = [
    'arrow',
    'boto3',
    'Click>=6.0',
    'epipearl>=0.2.0',
    'Jinja2',
    'json-spec',
    'python-dotenv',
    'requests'
]

test_requirements = [
    'pytest',
    'mock',
]

setup(
    name='epiclient',
    version=version,
    description="dce client for epipearl",
    long_description=readme + '\n\n' + history,
    author="nmaekawa",
    author_email='nmaekawa@g.harvard.edu',
    url='https://github.com/harvard-dce/epiclient',
    packages=find_packages(exclude=["docs", "tests*"]),
    package_dir={'epiclient':
                 'epiclient'},
    entry_points={
        'console_scripts': [
            'epiclient=epiclient.cli:main'
        ]
    },
    include_package_data=True,
    install_requires=requirements,
    license='Apache Software License 2.0',
    zip_safe=False,
    keywords='epiclient',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Natural Language :: English',
        "Programming Language :: Python :: 2",
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
    ],
    test_suite='tests',
    tests_require=test_requirements
)



# ````/usr/bin/env python
# -*- coding: utf-8 -*-

"""
test_epiclient
----------------------------------

Tests for `epiclient` module.
"""
import copy
import logging
from mock import call
from mock import MagicMock
import os
import pytest
import sys
import requests


import json

from epiclient.epiclient import Epiclient
from epipearl.epipearl import Epipearl
from epipearl.errors import SettingConfigError

from conftest import read_datafile


epiphan_url = "http://fake.example.edu"
epiphan_user = "user"
epiphan_passwd = "passwd"

# control skipping live tests according to command line option --runlive
# requires env vars EPI_URL, EPI_USER, EPI_PASSWD, EPI_PUBLISH_TYPE
livetest = pytest.mark.skipif(
        not pytest.config.getoption("--runlive"),
        reason=(
            "need --runlive option to run, plus env vars",
            "EPI_URL, EPI_USER, EPI_PASSWD"))

logging.basicConfig(stream=sys.stdout, level=logging.DEBUG)


class TestEpiclient(object):

    def setup(self):
        datastring = read_datafile('primary_sample_ca_settings')
        check = Epiclient.validate_json_input(datastring)
        self.primary_settings = json.loads(datastring)

        self.client = Epipearl(epiphan_url, epiphan_user, epiphan_passwd)


    def test_clear_channels_ok(self):
        mock = MagicMock(return_value=True)
        resp = Epiclient.clear_channels(mock, self.primary_settings)

        assert mock.delete_channel.call_count == 4
        assert mock.delete_recorder.call_count == 1
        mock.delete_channel.assert_any_call('1')
        mock.delete_channel.assert_any_call('2')
        mock.delete_channel.assert_any_call('3')
        mock.delete_channel.assert_any_call('4')
        mock.delete_recorder.assert_any_call('1')


    def test_create_channels_ok(self):

        def my_side_effect(*args, **kwargs):
            if args[0] == 'dce_pr':
                return '8'
            elif args[0] == 'dce_pn':
                return '10'
            elif args[0] == 'dce_live':
                return '11'
            elif args[0] == 'dce_live_lowbr':
                return '5'
            elif args[0] == 'dce_prpn':
                return '3'

        mock = MagicMock()
        mock.create_channel.side_effect = my_side_effect
        mock.create_recorder.side_effect = my_side_effect
        resp = Epiclient.create_channels(mock, self.primary_settings)

        assert mock.create_channel.call_count == 4
        mock.create_channel.assert_any_call('dce_pr')
        mock.create_channel.assert_any_call('dce_pn')
        mock.create_channel.assert_any_call('dce_live')
        mock.create_channel.assert_any_call('dce_live_lowbr')
        mock.create_recorder.assert_any_call('dce_prpn')
        assert self.primary_settings['channels'][
                'dce_pr']['channel_id'] == '8'
        assert self.primary_settings['channels'][
                'dce_pn']['channel_id'] == '10'
        assert self.primary_settings['channels'][
                'dce_live']['channel_id'] == '11'
        assert self.primary_settings['channels'][
                'dce_live_lowbr']['channel_id'] == '5'
        assert self.primary_settings['recorders'][
                'dce_prpn']['recorder_id'] == '3'


    def test_create_channels_error(self):

        def my_side_effect(*args, **kwargs):
            if args[0] == 'dce_pr':
                return '8'
            elif args[0] == 'dce_pn':
                return '10'
            elif args[0] == 'dce_live':
                return '11'
            elif args[0] == 'dce_live_lowbr':
                return '5'
            elif args[0] == 'dce_prpn':
                raise requests.HTTPError('another fake error')

        mock = MagicMock()
        mock.create_channel.side_effect = my_side_effect
        mock.create_recorder.side_effect = my_side_effect

        with pytest.raises(requests.HTTPError) as e:
            resp = Epiclient.create_channels(mock, self.primary_settings)
        assert 'another fake error' in e.value.message


    def test_configure_channel_layout_ok(self):
        pr_layout = read_datafile('sample_layout_pr')
        pn_layout = read_datafile('sample_layout_pn')
        live_layout = read_datafile('sample_layout_live')
        mock = MagicMock()
        mock.set_channel_layout.response_value = True

        resp = Epiclient.configure_channel_layouts(mock, self.primary_settings)

        assert mock.set_channel_layout.call_count == 4
        mock.set_channel_layout.assert_any_call(
                channel_id='1', layout=pr_layout)
        mock.set_channel_layout.assert_any_call(
                channel_id='2', layout=pn_layout)
        mock.set_channel_layout.assert_any_call(
                channel_id='3', layout=live_layout)
        mock.set_channel_layout.assert_any_call(
                channel_id='4', layout=live_layout)


    def test_configure_channel_encodings_ok(self):
        mock = MagicMock()
        mock.set_params.response_value = True

        resp = Epiclient.configure_channel_encodings(
                mock, self.primary_settings)

        assert mock.set_params.call_count == 4

        enc = copy.copy(self.primary_settings['channel_encodings'])
        enc.update(self.primary_settings['channels']['dce_pr']['encodings'])
        mock.set_params.assert_any_call(channel='1', params=enc)

        enc = copy.copy(self.primary_settings['channel_encodings'])
        enc.update(self.primary_settings['channels']['dce_pn']['encodings'])
        mock.set_params.assert_any_call(channel='2', params=enc)

        enc = copy.copy(self.primary_settings['channel_encodings'])
        enc.update(self.primary_settings['channels']['dce_live']['encodings'])
        mock.set_params.assert_any_call(channel='3', params=enc)

        enc = copy.copy(self.primary_settings['channel_encodings'])
        enc.update(self.primary_settings['channels'][
            'dce_live_lowbr']['encodings'])
        mock.set_params.assert_any_call(channel='4', params=enc)


    def test_configure_live_streaming_ok(self):
        mock = MagicMock()
        mock.set_channel_rtmp.return_value = True

        resp = Epiclient.configure_live_streaming(
                mock, self.primary_settings,
                rtmp_usr='soylent', rtmp_pwd='green')

        assert mock.set_channel_rtmp.call_count == 2

        mock.set_channel_rtmp.assert_any_call(
                channel_id='3',
                rtmp_url='rtmp://p.blobblob.i.streamingservice.org/EntryPoint',
                rtmp_stream='my-dev-epiphan',
                rtmp_usr='soylent',
                rtmp_pwd='green')

        mock.set_channel_rtmp.assert_any_call(
                channel_id='4',
                rtmp_url='rtmp://p.blobblob.i.streamingservice.org/EntryPoint',
                rtmp_stream='my-dev-epiphan',
                rtmp_usr='soylent',
                rtmp_pwd='green')


    def test_configure_general_settings_ok(self):
        mock = MagicMock()
        mock.return_value = True

        resp = Epiclient.configure_general_settings(mock, self.primary_settings)

        mock.set_ntp.assert_called_once()
        mock.set_ntp.assert_called_with(
                server='0.nz.pool.ntp.org', timezone='US/Alaska')
        mock.set_touchscreen.assert_called_once()
        mock.set_touchscreen.assert_called_with(screen_timeout=579)
        mock.set_permanent_logs.assert_called_once()
        mock.set_permanent_logs.assert_called_with(log_enabled=True)


    def test_configure_mhpearl_ok(self):
        mock = MagicMock()
        mock.return_value = True

        resp = Epiclient.configure_mhpearl(
                mock, self.primary_settings,
                mh_digest_usr='andromeda', mh_digest_pwd='strain')

        mock.set_mhpearl_settings.assert_called_once()
        mock.set_mhpearl_settings.assert_called_with(
                device_name='my-dev-epiphan',
                device_channel='1',
                file_search_range_in_seconds=58,
                admin_server_url="http://cluster.dev.url.edu",
                admin_server_usr='andromeda',
                admin_server_pwd='strain',
                update_frequency_in_seconds=123,
                backup_agent=False)


    def test_configure_deinterlacing_sources_ok(self):
        mock = MagicMock()
        mock.set_deinterlacing_source.return_value = True
        infocfg = {
                'channels': [
                    {'id': '1', 'name': 'dce_pr'},
                    {'id': '2', 'name': 'dce_live_lowbr'},
                    {'id': '3', 'name': 'dce_live'},
                    {'id': '4', 'name': 'dce_pn'}
                    ],
                'recorders': [
                    {'id': '1', 'name': 'dce_prpn'}
                    ],
                'sources': [
                    {'id': 'D12345678.hdmi-a', 'name': 'HDMI-A'},
                    {'id': 'D12345678.hdmi-b', 'name': 'HDMI-B'},
                    {'id': 'D12345678.sdi-a', 'name': 'SDI-A'},
                    {'id': 'D12345678.sdi-b', 'name': 'SDI-B'},
                    {'id': 'D12345678.vga-a', 'name': 'VGA-A'},
                    {'id': 'D12345678.vga-b', 'name': 'VGA-B'},
                    {'id': 'D12345678.analog-a', 'name': 'Analog-A Audio'},
                    {'id': 'D12345678.analog-b', 'name': 'Analog-B Audio'},
                    {'id': 'D12345678.hdmi-a-audio', 'name': 'HDMI-A Audio'},
                    {'id': 'D12345678.hdmi-b-audio', 'name': 'HDMI-B Audio'},
                    {'id': 'D12345678.sdi-a-audio', 'name': 'SDI-A Audio'},
                    {'id': 'D12345678.sdi-b-audio', 'name': 'SDI-B Audio'},
                    ]
                }

        resp = Epiclient.configure_deinterlacing_sources(
                mock, self.primary_settings, infocfg['sources'])

        expected = [
                call.set_deinterlacing(
                    source_name='D12345678.hdmi-a', deinterlacing=True),
                call.set_deinterlacing(
                    source_name='D12345678.hdmi-b', deinterlacing=True),
                call.set_deinterlacing(
                    source_name='D12345678.sdi-a', deinterlacing=True),
                call.set_deinterlacing(
                    source_name='D12345678.sdi-b', deinterlacing=True),
                call.set_deinterlacing(
                    source_name='D12345678.vga-a', deinterlacing=True),
                call.set_deinterlacing(
                    source_name='D12345678.vga-b', deinterlacing=True),
                ]
        mock.mock_calls == expected


    @livetest
    def test_live_configure_channel_layout(self):
        pr_layout = read_datafile('sample_layout_pr')
        pn_layout = read_datafile('sample_layout_pn')
        live_layout = read_datafile('sample_layout_live')

        ca_url = os.environ['EPI_URL']
        epi = Epipearl(
                ca_url, os.environ['EPI_USER'], os.environ['EPI_PASSWD'])

        resp = Epiclient.configure_channel_layouts(epi, self.primary_settings)

        assert resp == 'blu'
